/* Fetch list of newsgroups */
call RxFuncAdd 'SysLoadFuncs','RexxUtil','SysLoadFuncs'
call SysLoadFuncs
call RxFuncAdd 'SockLoadFuncs','RxSock','SockLoadFuncs'
call SockLoadFuncs 'q'

signal on HALT

parse arg host switch

if host=''|host='?'|host='/?' then do
    say ''
    say 'Usage: ngfetch.cmd <host> [/raw]'
    say ''
    say '<host> - hostname of news server'
    say ''
    say '/raw - write all received info without parsing'
    exit
end

if switch~translate='/RAW' then writeraw=1

crlf='0d0a'x
blocksize=10240
call SysCurState 'off'
con=.stream~new('stdout')
listend='2e0d0a'x

outfile=.stream~new('.newsrc')
call SysFileDelete outfile~qualify

port=119

sendstring='LIST'||crlf

csock=.socket~new

say 'Connecting to host '||host||' on port '||port||'...'
check=csock~connect(host||':'||port)
if check=-1 then do
    say 'Connection to host '||host||' on port '||port||' failed!'
    csock~shutdown
    csock~close
    exit
end
say ''

say 'Requesting group list...'
check=csock~send(sendstring)
if check=-1 then do
    say 'Failed to send group request!'
    csock~shutdown
    csock~close
    exit
end
say ''

say 'Receiving group list... '
say ''
parse value SysCurPos() with row col
total=0
calc=0
next=0
data=''
freq=10240
freq2=1024
call time 'r'
check=csock~receive(blocksize)
total=total+check[1]~length
if writeraw=1 then outfile~charout(check[1])
do until check[1]~right(3)=listend
    check=csock~receive(blocksize)
    call SysCurPos row,0
    con~charout('Bytes received: ')
    call SysCurPos row,32
    con~charout('Bytes/second: ')
    rvar=check[2]
    data=data||check[1]
    total=total+data~length
    if total>=next | data~right(3)=listend then do
        ct=time('e')
        next=total+freq2
        call SysCurPos row,16
        con~charout(codel(total)~left(12))
        if total>50000 then freq2=rate/2
    end
    if total>=calc | data~right(3)=listend then do
        rate=total%ct
        calc=total+freq
        if total>50000 then freq=rate/2
        call SysCurPos row,46
        con~charout(codel(rate)~left(12))
    end
    if writeraw=1 then do
        outfile~charout(data)
        data=''
    end
    else do
        if data~word(1)='215' then do
            parse var data '215' . (crlf) data
        end
        do while data~pos(crlf)>0
            parse var data group ' ' (crlf) data
            outfile~lineout(group)
            if data~left(3)=listend then leave
        end        
    end
end 
outfile~close 
say ''
say ''

say 'Shutting down socket...'
csock~shutdown('both')
say 'Closing socket...'
csock~close
say 'Done!'
exit

HALT:

say ''
say ''
say 'Aborting!'
if csock~objectname='a socket' then do
    say ''
    say 'Shutting down socket...'
    csock~shutdown('both')
    say 'Closing socket...'
    csock~close
    say 'Done!'
end
exit

/* Socket class - work in progress **********************************/
/*                                                                  */
::class 'socket'

::method init
expose portnum
portnum=SockSocket('AF_INET','SOCK_STREAM',0)
return

::method sock
expose portnum
return portnum

::method connect
use arg servname
parse var servname hostname ':' host.!port
parse var hostname o1 '.' o2 '.' o3 '.' o4
if o1~datatype='NUM' & o2~datatype='NUM' & o3~datatype='NUM' &,
   o4~datatype='NUM' then do
    if o1~datatype('w')=1 & o2~datatype('w')=1 & o3~datatype('w')=1 &,
       o4~datatype('w')=1 then do
        if (o1>=0 & o1<=255) & (o2>=0 & o2<=255) & (o3>=0 & o3<=255) &,
           (o4>=0 & o4<=255) then do
            stype='IP'
            host.!addr=hostname
        end
    end
end
else stype='NAME'            
if stype='NAME' then call SockGetHostByName hostname,'host.!'
host.!family='AF_INET'
rc=SockConnect(self~sock,'host.!')
return rc

::method send
use arg sendstring
rc=SockSend(self~sock,sendstring)
return rc

::method receive
use arg length
if length='LENGTH' then length=1024
check=SockRecv(self~sock,'sockdata',length)
return .array~new~~put(sockdata,1)~~put(check,2)

::method shutdown
use arg how
select
    when how~translate='FROM' then how=0
    when how~translate='TO' then how=1
    otherwise how=2
end
call SockShutDown self~sock,how
return

::method close
call SockClose self~sock
return
/*                                                                  */
/********************************************************************/
